// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef LIBKLEO_VERSION_H
#define LIBKLEO_VERSION_H

#define LIBKLEO_VERSION_STRING "5.7.3"
#define LIBKLEO_VERSION_MAJOR 5
#define LIBKLEO_VERSION_MINOR 7
#define LIBKLEO_VERSION_PATCH 3
#define LIBKLEO_VERSION ((5<<16)|(7<<8)|(3))

#endif
